__version__ = '0.5.post3'
__author__ = 'Outernet Inc'
__copyright__ = """
Copyright 2015, Outernet Inc.
Some rights reserved.

This software is free software licensed under the terms
of GPLv3. See COPYING file that comes with the source code,
or http://www.gnu.org/licenses/gpl.txt.
"""
